/*
************************************************************************
*
*   ExStyle.c - display style setting commands
*
*   Copyright (c) 1994
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 95/11/24
*   Pathname of SCCS file     : /sgiext/molmol/src/cmdattr/SCCS/s.ExStyle.c
*   SCCS identification       : 1.7
*
************************************************************************
*/

#include <cmd_attr.h>

#include <stdio.h>

#include <arg.h>
#include <prop_tab.h>
#include <data_hand.h>
#include <attr_struc.h>
#include <attr_mng.h>
#include <graph_draw.h>

static int ChoiceAtom = 0, ChoiceBond = 1, ChoiceDist = 1;
static PropRefP VisibleRefP;

static void
setAtomStyle(DhAtomP atomP, void *clientData)
{
  AtomStyle style = *(AtomStyle *) clientData;
  AttrP attrP;
  struct AttrS attr;

  attrP = DhAtomGetAttr(atomP);
  AttrCopy(&attr, attrP);
  AttrReturn(attrP);
  attr.atomStyle = style;
  if (style == AS_SPHERE)
    attr.shadeModel = SHADE_DEFAULT;
  else
    attr.shadeModel = -1;
  DhAtomSetAttr(atomP, AttrGet(&attr));

  DhAtomSetProp(VisibleRefP, atomP, style != AS_INVISIBLE);
}

static void
setBondStyle(DhBondP bondP, DhAtomP atom1P, DhAtomP atom2P, void *clientData)
{
  BondStyle style = *(BondStyle *) clientData;
  AttrP attrP;
  struct AttrS attr;

  attrP = DhBondGetAttr(bondP);
  AttrCopy(&attr, attrP);
  AttrReturn(attrP);
  attr.bondStyle = style;
  if (style == BS_CYLINDER || style == BS_HALF_CYLINDER ||
      style == BS_NEON || style == BS_HALF_NEON)
    attr.shadeModel = SHADE_DEFAULT;
  else
    attr.shadeModel = -1;
  DhBondSetAttr(bondP, AttrGet(&attr));

  DhBondSetProp(VisibleRefP, bondP, style != BS_INVISIBLE);
}

static void
setDistStyle(DhDistP distP, DhAtomP atom1P, DhAtomP atom2P, void *clientData)
{
  DistStyle style = *(DistStyle *) clientData;
  AttrP attrP;
  struct AttrS attr;

  attrP = DhDistGetAttr(distP);
  AttrCopy(&attr, attrP);
  AttrReturn(attrP);
  attr.distStyle = style;
  if (style == DS_CYLINDER || style == DS_NEON)
    attr.shadeModel = SHADE_DEFAULT;
  else
    attr.shadeModel = -1;
  DhDistSetAttr(distP, AttrGet(&attr));

  DhDistSetProp(VisibleRefP, distP, style != DS_INVISIBLE);
}

#define ARG_NUM 1

ErrCode
ExStyleAtom(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enumEntry[3];
  ErrCode errCode;
  AtomStyle style;

  arg[0].type = AT_ENUM;

  ArgInit(arg, ARG_NUM);

  enumEntry[0].str = "invisible";
  enumEntry[0].onOff = FALSE;
  enumEntry[1].str = "sphere";
  enumEntry[1].onOff = FALSE;
  enumEntry[2].str = "tetrahedron";
  enumEntry[2].onOff = FALSE;

  enumEntry[ChoiceAtom].onOff = TRUE;

  arg[0].prompt = "Atom Style";
  arg[0].u.enumD.entryP = enumEntry;
  arg[0].u.enumD.n = 3;
  arg[0].v.intVal = ChoiceAtom;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  ChoiceAtom = arg[0].v.intVal;
  ArgCleanup(arg, ARG_NUM);

  if (ChoiceAtom == 0)
    style = AS_INVISIBLE;
  else if (ChoiceAtom == 1)
    style = AS_SPHERE;
  else
    style = AS_TETRAHEDRON;

  VisibleRefP = PropGetRef(PROP_VISIBLE, FALSE);
  DhApplyAtom(PropGetRef(PROP_SELECTED, FALSE), setAtomStyle, &style);

  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}

ErrCode
ExStyleBond(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enumEntry[7];
  ErrCode errCode;
  BondStyle style;

  arg[0].type = AT_ENUM;

  ArgInit(arg, ARG_NUM);

  enumEntry[0].str = "invisible";
  enumEntry[0].onOff = FALSE;
  enumEntry[1].str = "line";
  enumEntry[1].onOff = FALSE;
  enumEntry[2].str = "half_line";
  enumEntry[2].onOff = FALSE;
  enumEntry[3].str = "cylinder";
  enumEntry[3].onOff = FALSE;
  enumEntry[4].str = "half_cylinder";
  enumEntry[4].onOff = FALSE;
  enumEntry[5].str = "neon";
  enumEntry[5].onOff = FALSE;
  enumEntry[6].str = "half_neon";
  enumEntry[6].onOff = FALSE;

  enumEntry[ChoiceBond].onOff = TRUE;

  arg[0].prompt = "Bond Style";
  arg[0].u.enumD.entryP = enumEntry;
  arg[0].u.enumD.n = 7;
  arg[0].u.enumD.lineNo = 2;
  arg[0].v.intVal = ChoiceBond;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  ChoiceBond = arg[0].v.intVal;
  ArgCleanup(arg, ARG_NUM);

  if (ChoiceBond == 0)
    style = BS_INVISIBLE;
  else if (ChoiceBond == 1)
    style = BS_LINE;
  else if (ChoiceBond == 2)
    style = BS_HALF_LINE;
  else if (ChoiceBond == 3)
    style = BS_CYLINDER;
  else if (ChoiceBond == 4)
    style = BS_HALF_CYLINDER;
  else if (ChoiceBond == 5)
    style = BS_NEON;
  else
    style = BS_HALF_NEON;

  VisibleRefP = PropGetRef(PROP_VISIBLE, FALSE);
  DhApplyBond(PropGetRef(PROP_SELECTED, FALSE), setBondStyle, &style);

  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}

ErrCode
ExStyleDist(char *cmd)
{
  ArgDescr arg[ARG_NUM];
  EnumEntryDescr enumEntry[5];
  ErrCode errCode;
  DistStyle style;

  arg[0].type = AT_ENUM;

  ArgInit(arg, ARG_NUM);

  enumEntry[0].str = "invisible";
  enumEntry[0].onOff = FALSE;
  enumEntry[1].str = "line";
  enumEntry[1].onOff = FALSE;
  enumEntry[2].str = "viol";
  enumEntry[2].onOff = FALSE;
  enumEntry[3].str = "cylinder";
  enumEntry[3].onOff = FALSE;
  enumEntry[4].str = "neon";
  enumEntry[4].onOff = FALSE;

  enumEntry[ChoiceDist].onOff = TRUE;

  arg[0].prompt = "Dist Style";
  arg[0].u.enumD.entryP = enumEntry;
  arg[0].u.enumD.n = 5;
  arg[0].u.enumD.lineNo = 2;
  arg[0].v.intVal = ChoiceDist;

  errCode = ArgGet(arg, ARG_NUM);
  if (errCode != EC_OK) {
    ArgCleanup(arg, ARG_NUM);
    return errCode;
  }

  ChoiceDist = arg[0].v.intVal;
  ArgCleanup(arg, ARG_NUM);

  if (ChoiceDist == 0)
    style = DS_INVISIBLE;
  else if (ChoiceDist == 1)
    style = DS_LINE;
  else if (ChoiceDist == 2)
    style = DS_VIOL;
  else if (ChoiceDist == 3)
    style = DS_CYLINDER;
  else
    style = DS_NEON;

  VisibleRefP = PropGetRef(PROP_VISIBLE, FALSE);
  DhApplyDist(PropGetRef(PROP_SELECTED, FALSE), setDistStyle, &style);

  GraphMolChanged(PROP_SELECTED);
  GraphRedrawNeeded();

  return EC_OK;
}
